==> synpay-8.x-1.9.tar.gz: synpay/src/Plugin/Synpay/SynpayYooKassa.php <==
<?php

namespace Drupal\synpay\Plugin\Synpay;

use Ramsey\Uuid\Uuid;
use YooKassa\Client;
use YooKassa\Model\PaymentStatus;
use YooKassa\Model\NotificationEventType;
use YooKassa\Model\Notification\NotificationSucceeded;
use YooKassa\Request\Payments\Payment\CreateCaptureRequest;
use YooKassa\Model\Notification\NotificationWaitingForCapture;
use Drupal\Component\Serialization\Json;
use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Routing\TrustedRedirectResponse;

/**
 * Provides a Yandex Gateway.
 *
 * @SynpayAnnotation(
 *   id = "yookassa",
 *   title = @Translation("Ykassa"),
 * )
 */
class SynpayYooKassa extends SynpayPluginBase implements SynpayPluginInterface {
  // TRUE - цена с копейками, FALSE - цена в копейках.
  const PRECISION = TRUE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    // $this->client = NULL;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $this->client = new Client();
    $this->client->setAuth($this->user, $this->pass);

  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $return = "$host/synpay/return_direct/yookassa";
    $order = Uuid::uuid4()->getHex()->toString();
    try {
      $response = $this->client->createPayment([
        'amount' => [
          'value' => $total,
          'currency' => 'RUB',
        ],
        'payment_method_data' => [
          'type' => 'bank_card',
        ],
        'confirmation' => [
          'type' => 'redirect',
          'return_url' => $return,
        ],
        'capture' => TRUE,
        'description' => "Заказ №$order",
      ],
      uniqid('', TRUE),
      );
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_yookassa')->error($message);
      return FALSE;
    }
    $url = $response->getConfirmation()->getConfirmationUrl();
    $orderId = substr($url, strpos($url, 'orderId=') + 8);
    $session = \Drupal::request()->getSession();
    $session->set('orderId', $orderId);
    $redirect = new TrustedRedirectResponse($response->getConfirmation()->getConfirmationUrl());
    $redirect->send();
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    try {
      $response = $this->client->createPayment([
        'amount' => [
          'value' => $order_amount,
          'currency' => 'RUB',
        ],
        'payment_method_data' => [
          'type' => 'bank_card',
        ],
        'confirmation' => [
          'type' => 'redirect',
          'return_url' => $params['returnUrl'],
        ],
        'capture' => TRUE,
        'description' => "Заказ №$order_id",
      ],
      uniqid('', TRUE),
      );
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_yookassa')->error($message);
      return FALSE;
    }
    return [
      'orderId' => $response->getId(),
      'formUrl' => $response->getConfirmation()->getConfirmationUrl(),
    ];
  }

  /**
   * Pay callback.
   */
  public function callback(Request $request) {
    $session = \Drupal::request()->getSession();
    $orderId = $session->get('orderId');
    $payment = $this->client->getPaymentInfo($orderId);
    switch ($payment->getStatus()) {
      case 'succeeded':
        $result['status'] = 'completed';
        break;

      default:
        $result['status'] = 'any';
        break;
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onReturn(string $remote_id) {
    $paymentInfoResponse = $this->client->getPaymentInfo($remote_id);
    switch ($paymentInfoResponse->status) {
      case PaymentStatus::SUCCEEDED:
        $result['status'] = 'completed';
        break;

      default:
        $result['status'] = 'authorization_voided';
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $result = $this->client->getOrderStatus($remote_id);
    $rawBody = $request->getContent();
    \Drupal::logger(__FUNCTION__ . __LINE__)->notice(
      '@j', ['@j' => json_encode($rawBody ?? '')]
    );
    $this->log('Notification: ' . $rawBody);
    $notificationData  = Json::decode($rawBody);
    $notificationModel = ($notificationData['event'] === NotificationEventType::PAYMENT_SUCCEEDED)
            ? new NotificationSucceeded($notificationData)
            : new NotificationWaitingForCapture($notificationData);
    $apiClient         = $this->apiClient;
    $paymentResponse   = $notificationModel->getObject();
    $paymentId         = $paymentResponse->id;
    $payment_storage   = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments          = $payment_storage->loadByProperties(['remote_id' => $paymentId]);
    if (!$payments) {
      return new Response('Bad request', 400);
    }
    /** @var \Drupal\commerce_payment\Entity\Payment $payment */
    $payment = reset($payments);
    /** @var \Drupal\commerce_order\Entity\Order $order */
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }

    $paymentInfo = $apiClient->getPaymentInfo($paymentId);
    $this->log('Payment info: ' . Json::encode($paymentInfo));

    $state = $order->getState()->value;
    if ($state !== 'completed') {
      switch ($paymentInfo->status) {
        case PaymentStatus::WAITING_FOR_CAPTURE:
          $captureRequest  = CreateCaptureRequest::builder()->setAmount($paymentInfo->getAmount())->build();
          $captureResponse = $apiClient->capturePayment($captureRequest, $paymentId);
          $this->log('Payment info after capture: ' . Json::encode($captureResponse));
          if ($captureResponse->status == PaymentStatus::SUCCEEDED) {
            $payment->setRemoteState($paymentInfo->status);
            $order->state = 'completed';
            $order->setCompletedTime(\Drupal::time()->getRequestTime());
            $order->save();
            $payment->save();
            $this->log('Payment completed');

            return new Response('Payment completed', 200);
          }
          elseif ($captureResponse->status == PaymentStatus::CANCELED) {
            $payment->setRemoteState($paymentInfo->status);
            $payment->save();
            $this->log('Payment canceled');

            return new Response('Payment canceled', 200);
          }
          break;

        case PaymentStatus::PENDING:
          $payment->setRemoteState($paymentInfo->status);
          $payment->save();
          $this->log('Payment pending');

          return new Response(' Payment Required', 402);

        case PaymentStatus::SUCCEEDED:
          $payment->setRemoteState($paymentInfo->status);
          $order->state = 'completed';
          $order->setCompletedTime(\Drupal::time()->getRequestTime());
          $order->save();
          $payment->save();
          $this->log('Payment complete');

          return new Response('Payment complete', 200);

        case PaymentStatus::CANCELED:
          $payment->setRemoteState($paymentInfo->status);
          $payment->save();
          $this->log('Payment canceled');

          return new Response('Payment canceled', 200);
      }
    }
    return $result;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayYandex')->info($message);
  }

}

==> synpay-8.x-1.13.tar.gz: synpay/src/Plugin/Synpay/SynpayYooKassa.php <==
<?php

namespace Drupal\synpay\Plugin\Synpay;

use Ramsey\Uuid\Uuid;
use YooKassa\Client;
use Drupal\commerce_order\Event\OrderEvent;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_checkout\Event\CheckoutEvents;
use Drupal\Component\Serialization\Json;
use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\Core\Routing\TrustedRedirectResponse;

/**
 * Provides a Yandex Gateway.
 *
 * @SynpayAnnotation(
 *   id = "yookassa",
 *   title = @Translation("Ykassa"),
 * )
 */
class SynpayYooKassa extends SynpayPluginBase implements SynpayPluginInterface {
  // TRUE - цена с копейками, FALSE - цена в копейках.
  const PRECISION = TRUE;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    // $this->client = NULL;
    $this->order = isset($configuration['order']) ? $configuration['order'] : FALSE;
    $this->total = isset($configuration['total']) ? $configuration['total'] : FALSE;
    $this->lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $this->client = new Client();
    $this->client->setAuth($this->user, $this->pass);

  }

  /**
   * {@inheritdoc}
   */
  public function pay(string $order, int $total) {
    if ($order != $this->order) {
      \Drupal::messenger()->addError(t("Wrong order number"));
      return;
    }
    $host = \Drupal::request()->getSchemeAndHttpHost();
    $return = "$host/synpay/return_direct/yookassa";
    $order = Uuid::uuid4()->getHex()->toString();
    try {
      $response = $this->client->createPayment([
        'amount' => [
          'value' => $total,
          'currency' => 'RUB',
        ],
        'payment_method_data' => [
          'type' => 'bank_card',
        ],
        'confirmation' => [
          'type' => 'redirect',
          'return_url' => $return,
        ],
        'capture' => TRUE,
        'description' => "Заказ №$order",
      ],
      uniqid('', TRUE),
      );
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_yookassa')->error($message);
      return FALSE;
    }
    $url = $response->getConfirmation()->getConfirmationUrl();
    $orderId = substr($url, strpos($url, 'orderId=') + 8);
    $session = \Drupal::request()->getSession();
    $session->set('orderId', $orderId);
    $redirect = new TrustedRedirectResponse($response->getConfirmation()->getConfirmationUrl());
    $redirect->send();
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, $order_amount, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    try {
      $response = $this->client->createPayment([
        'amount' => [
          'value' => $order_amount,
          'currency' => 'RUB',
        ],
        'payment_method_data' => [
          'type' => 'bank_card',
        ],
        'confirmation' => [
          'type' => 'redirect',
          'return_url' => $params['returnUrl'],
        ],
        'capture' => TRUE,
        'description' => "Заказ №$order_id",
      ],
      uniqid('', TRUE),
      );
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_yookassa')->error($message);
      return FALSE;
    }
    return [
      'orderId' => $response->getId(),
      'formUrl' => $response->getConfirmation()->getConfirmationUrl(),
    ];
  }

  /**
   * Pay callback.
   */
  public function callback(Request $request) {
    $session = \Drupal::request()->getSession();
    $orderId = $session->get('orderId');
    $payment = $this->client->getPaymentInfo($orderId);
    switch ($payment->getStatus()) {
      case 'succeeded':
        $result['status'] = 'completed';
        break;

      default:
        $result['status'] = 'any';
        break;
    }
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['order_id' => $order->id()]);
    $success = FALSE;
    foreach ($payments as $payment) {
      if (!empty($payment->remote_id->value)) {
        $status = $this->checkPayment($payment->remote_id->value);
        $answer = $this->updatePayment($payment, $status);
        if ($answer == 'completed' && !$success) {
          $success = TRUE;
        }
      }
    }
    $result = $success ? 'completed' : $answer;
    return $result;
  }

  /**
   * Commerce Return.
   */
  public function onNotify(Request $request) {
    $rawBody = $request->getContent();
    $this->log('Notification: ' . $rawBody);
    $paymentInfo = Json::decode($rawBody);
    $payment_storage = \Drupal::entityTypeManager()->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties(['remote_id' => $paymentInfo['object']['id']]);
    $payment = reset($payments);
    if (!$payment) {
      return new Response('Bad request', 400);
    }
    $order = $payment->getOrder();
    if (!$order) {
      return new Response('Order not found', 404);
    }
    if ($paymentInfo['object']['paid']) {
      $payment->state = 'completed';
      $payment->setCompletedTime(time());
      if ($order->state->value != 'completed') {
        $event = new OrderEvent($order);
        $eventDispatcher = \Drupal::service('event_dispatcher');
        $eventDispatcher->dispatch(CheckoutEvents::COMPLETION, $event);
        $order->getState()->applyTransitionById('place');
        $order->save();
      }
    }
    else {
      $payment->state = 'authorization_voided';
    }
    $payment->setRemoteState($paymentInfo['object']['status']);
    $payment->save();

    return new Response('OK', 200);
  }

  /**
   * Check Payments.
   */
  private function checkPayment($id) {
    try {
      $response = $this->client->getPaymentInfo($id);
      return $response;
    }
    catch (ClientException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::logger('synpay_yookassa')->error($message);
      return FALSE;
    }
    return FALSE;
  }

  /**
   * UpdatePayment.
   */
  private function updatePayment($payment, $status) {
    if (!$status) {
      $payment->state = 'authorization_voided';
      $payment->setRemoteState('Error');
      $result = 'authorization_voided';
    }
    elseif ($status->getPaid()) {
      $payment->state = 'completed';
      $payment->setCompletedTime(time());
      $payment->setRemoteState($status->getStatus());
      $result = 'completed';
    }
    else {
      $payment->state = 'authorization_voided';
      $payment->setRemoteState($status->getStatus());
      $result = 'authorization_voided';
    }
    $payment->save();
    return $result;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayYandex')->info($message);
  }

}
